;	======================================
;	       Sensibelchen mit Tiny13
;	======================================

;	------------- Port usage -------------
;	B0: Output LEDs Cold White
;	B1: Output LEDs Warm White
;	B2: Output LEDs Violet(UV)
;	B3: Input Mode Select
;	B4: Input Passive IR Sensor

;	------------- Timer usage ------------
;	Timer0: Interrupt every 0,0002 s (Prescaler CK/8)

;	-- Standard Definitions for ATtiny13 -
	.include "tn13def.inc"

;	------------- SRAM usage -------------
.dseg
.org	0x60

;	------------- Constants --------------
	.equ	rambeg	=0x60		;RAM start
	.equ	cwaitl	=200		;value long waiting loop
	.equ	cwaitm	=90		;value medium waiting loop
	.equ	cwaitms	=50		;value medium short waiting loop
	.equ	cwaits	=25		;value short waiting loop
	.equ	pwmmax	=64		;max PWM pulses
	.equ	promax	=5		;max Program number
	.equ	port	=portb
	.equ	pin	=pinb
	.equ	ddr	=ddrb
	.equ	olcw	=0
	.equ	olww	=1
	.equ	olvi	=2
	.equ	imode	=3
	.equ	isens	=4

;	-------- Register definitions --------
	.def	stor	=r0		;storage variable for LPM
	.def	flags	=r1		;flags storage register
	.def	iflags	=r2		;input flags
	.def	temp	=r16		;temporary variable
	.def	temp2	=r17		;temporary variable 2
	.def	pwmset	=r18		;PWM Setpoint
	.def	pwm	=r19		;PWM Counter
	.def	waitco	=r20		;wait loop counter
	.def	loop	=r21		;loop counter
	.def	relco1	=r22		;relax counter 1
	.def	relco2	=r23		;relax counter 2
	.def	proco	=r24		;program counter
	.def	led1	=r25
	.def	led2	=r26
	.def	led3	=r27

;	- Program beginn, Interrupt Vectors --
.cseg
.org	0x0000

reset:	rjmp	initia			;Programmstart
exint0:	reti
pncnge:	reti
t0ovfl:	rjmp	timer0
eerdy:	reti
ancomp:	reti
t0coma:	reti
t0comb:	reti
wdtto:	reti
adccc:	reti

;	Initialisierung von RAM, I/O und Interrupts

initia:	cli				;Clear Interrupts
;	ldi	temp,0x62
;	out	osccal,temp		;Callibrate Oscillator (Tiny13: auto)
	ldi	temp,0x07
	out	ddr,temp		;Inputs & Outputs
	ldi	temp,0x18
	out	port,temp		;Set Pull-Ups
	ldi	temp,low(ramend)
	out	spl,temp		;Set Stack Pointer
	ldi	temp,0
	ldi	yl,low(rambeg)		;Beginn des RAM-Bereiches Low
	ldi	yh,high(rambeg)		;Beginn des RAM-Bereiches High
init:	st	y+,temp			;Speicherzelle loeschen
	cpi	yl,low(ramend+1)	;RAM-Ende (Low) erreicht?
	brne	init			;nein -> Schleife
	cpi	yh,high(ramend+1)	;RAM-Ende (High) erreicht?
	brne	init			;nein -> Schleife
	ldi	temp,1<<toie0
	out	timsk0,temp		;Timer0 Interrupt
	ldi	temp,0
	out	tccr0a,temp
	ldi	temp,1<<cs01
	out	tccr0b,temp		;Timer0 Overflow, prescaler 256
	sei				;start interrupts
	
	clr	proco
	clr	pwmset
	clr	loop

main:	sbic	pin,imode
	rjmp	main2

; -------- Main program 1 ---------

	clr	loop
	clr	pwmset
main1:	cpi	proco,0
	brne	m11
	rjmp	m1p1
m11:	cpi	proco,1
	brne	m12
	rjmp	m1p2
m12:	cpi	proco,2
	brne	m13
	rjmp	m1p3
m13:	cpi	proco,3
	brne	m14
	rjmp	m1p4
m14:	cpi	proco,4
	brne	m15
	rjmp	m1p5
m15:	cpi	proco,5
	brne	main1
	rjmp	m1p6

m1p1:	cpi	loop,0
	brne	m112a
	clr	led1
	clr	led2
	mov	led3,pwmset
	cpi	pwmset,pwmmax
	breq	m112
	inc	pwmset
	rjmp	m112w

m112:	inc	loop
	ldi	pwmset,pwmmax
m112a:	cpi	loop,1
	brne	m112w
	clr	led1
	clr	led2
	mov	led3,pwmset
	cpi	pwmset,0
	breq	m112r
	dec	pwmset

m112w:	rcall	waitl
	rjmp	m1p1
m112r:	rcall	waitl
	rjmp	main

m1p2:	cpi	loop,0
	brne	m122a
	clr	led1
	mov	led2,pwmset
	clr	led3
	cpi	pwmset,pwmmax/2
	breq	m122
	inc	pwmset
	rjmp	m122w

m122:	inc	loop
	ldi	pwmset,pwmmax/2
m122a:	cpi	loop,1
	brne	m122w
	clr	led1
	mov	led2,pwmset
	clr	led3
	cpi	pwmset,0
	breq	m122r
	dec	pwmset

m122w:	rcall	waitl
	rjmp	m1p2
m122r:	rcall	waitl
	rjmp	main

m1p3:	cpi	loop,0
	brne	m132a
	clr	led1
	mov	led2,pwmset
	clr	led3
	cpi	pwmset,pwmmax
	breq	m132
	inc	pwmset
	rjmp	m132w

m132:	inc	loop
	ldi	pwmset,pwmmax
m132a:	cpi	loop,1
	brne	m132w
	clr	led1
	mov	led2,pwmset
	clr	led3
	cpi	pwmset,0
	breq	m132r
	dec	pwmset

m132w:	rcall	waitm
	rjmp	m1p3
m132r:	rcall	waitm
	rjmp	main

m1p4:	cpi	loop,0
	brne	m142a
	mov	led1,pwmset
	clr	led2
	clr	led3
	cpi	pwmset,pwmmax
	breq	m142
	inc	pwmset
	rjmp	m142w

m142:	inc	loop
	ldi	pwmset,pwmmax
m142a:	cpi	loop,1
	brne	m142w
	mov	led1,pwmset
	clr	led2
	clr	led3
	cpi	pwmset,0
	breq	m142r
	dec	pwmset

m142w:	rcall	waitms
	rjmp	m1p4
m142r:	rcall	waitms
	rjmp	main

m1p5:	cpi	loop,0
	brne	m152a
	mov	led1,pwmset
	mov	led2,pwmset
	clr	led3
	cpi	pwmset,pwmmax
	breq	m152
	inc	pwmset
	rjmp	m152w

m152:	inc	loop
	ldi	pwmset,pwmmax
m152a:	cpi	loop,1
	brne	m152w
	mov	led1,pwmset
	mov	led2,pwmset
	clr	led3
	cpi	pwmset,0
	breq	m152r
	dec	pwmset

m152w:	rcall	waits
	rjmp	m1p5
m152r:	rcall	waits
	rjmp	main

m1p6:	cpi	loop,0
	brne	m162a
	mov	led1,pwmset
	ldi	led2,pwmmax
	clr	led3
	cpi	pwmset,pwmmax
	brsh	m162
	inc	pwmset
	inc	pwmset
	rjmp	m162w

m162:	inc	loop
	ldi	pwmset,pwmmax
m162a:	cpi	loop,1
	brne	m162w
	mov	led1,pwmset
	ldi	led2,pwmmax
	clr	led3
	cpi	pwmset,0
	breq	m162r
	dec	pwmset
	dec	pwmset

m162w:	rcall	waits
	rjmp	m1p6
m162r:	rcall	waits
	rjmp	main

; -------- Main program 2 ---------

main2:	cpi	loop,0
	brne	m212a
	mov	led1,pwmset
	clr	led2
	clr	led3
	cpi	pwmset,pwmmax
	breq	m212
	inc	pwmset
	rjmp	m2end

m212:	inc	loop
	ldi	pwmset,pwmmax
m212a:	cpi	loop,1
	brne	m221a
	mov	led1,pwmset
	clr	led2
	clr	led3
	cpi	pwmset,0
	breq	m221
	dec	pwmset
	rjmp	m2end

m221:	inc	loop
	clr	pwmset
m221a:	cpi	loop,2
	brne	m222a
	clr	led1
	mov	led2,pwmset
	clr	led3
	cpi	pwmset,pwmmax
	breq	m222
	inc	pwmset
	rjmp	m2end

m222:	inc	loop
	ldi	pwmset,pwmmax
m222a:	cpi	loop,3
	brne	m231a
	clr	led1
	mov	led2,pwmset
	clr	led3
	cpi	pwmset,0
	breq	m231
	dec	pwmset
	rjmp	m2end

m231:	inc	loop
	clr	pwmset
m231a:	cpi	loop,4
	brne	m232a
	clr	led1
	clr	led2
	mov	led3,pwmset
	cpi	pwmset,pwmmax
	breq	m232
	inc	pwmset
	rjmp	m2end

m232:	inc	loop
	ldi	pwmset,pwmmax
m232a:	cpi	loop,5
	brne	m2end
	clr	led1
	clr	led2
	mov	led3,pwmset
	cpi	pwmset,0
	breq	m2ve
	dec	pwmset
	rjmp	m2end

m2ve:	clr	loop
	clr	pwmset

m2end:	rcall 	waitl
	rjmp	main

;	--------- Long Waiting loop ----------
waitl:	clr	waitco
wal1:	sbic	pin,isens
	cpi	waitco,cwaitl
	sbis	pin,isens
	cpi	waitco,cwaitl/2
	brlo	wal1
	ret

;	--------- Medium Waiting loop --------
waitm:	clr	waitco
wam1:	cpi	waitco,cwaitm
	brlo	wam1
	ret

;	--------- Medium Short Waiting loop --------
waitms:	clr	waitco
wams1:	cpi	waitco,cwaitms
	brlo	wams1
	ret

;	--------- Short Waiting loop ---------
waits:	clr	waitco
was1:	cpi	waitco,cwaits
	brlo	was1
	ret

;	------------ Read EEPROM -------------
eeread:	cli
	sbic	eecr,eewe
	rjmp	eeread
	out	eearl,temp
	sbi	eecr,eere
	in	pwmset,eedr
	sei
	ret

;	------------ Write EEPROM ------------
eewrit:	cli
	sbic	eecr,eewe
	rjmp	eewrit
	out	eearl,temp
	out	eedr,temp2
	sbi	eecr,eemwe
	sbi	eecr,eewe
	sei
	ret

;	------------- Timer0 ISR -------------
timer0:	in	flags,sreg	;save flags
	inc	waitco

	sbic	pin,isens	;falling edge detection
	rjmp	tim2		;isens is set
	sbrs	iflags,isens
	rjmp	tim2		;iflag is set
	cpi	proco,promax
	brsh	tim2
	inc	proco
	clr	relco2
tim2:	in	iflags,pin	;save input

tlcw:	cpi	led1,0
	breq	tlcw0
	cpi	led1,pwmmax
	breq	tlcw1
	cp	pwm,led1
	brlo	tlcw1
tlcw0:	cbi	port,olcw
	rjmp	tlww
tlcw1:	sbi	port,olcw

tlww:	cpi	led2,0
	breq	tlww0
	cpi	led2,pwmmax
	breq	tlww1
	cp	pwm,led2
	brlo	tlww1
tlww0:	cbi	port,olww
	rjmp	tlvi
tlww1:	sbi	port,olww

tlvi:	cpi	led3,0
	breq	tlvi0
	cpi	led3,pwmmax
	breq	tlvi1
	cp	pwm,led3
	brlo	tlvi1
tlvi0:	cbi	port,olvi
	rjmp	tpwm
tlvi1:	sbi	port,olvi

tpwm:	inc	pwm		;increment PWM counter
	cpi	pwm,pwmmax
	brlo	tpwm2
	clr	pwm		;reset PWM counter
tpwm2:	nop

	inc	relco1		;decrement program counter
	cpi	relco1,255
	brlo	tend
	clr	relco1
	inc	relco2
	cpi	relco2,255
	brlo	tend
	clr	relco2
	cpi	proco,0
	breq	tend
	dec	proco

tend:	out	sreg,flags	;restore flags
	reti
